# RemoteBox and VirtualBox preferences
use strict;
use warnings;
our %gui;
# OLD RDP: rdesktop -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p
our %prefs = (RDPCLIENT       => 'xfreerdp -g %Xx%Y --plugin cliprdr --plugin rdpsnd -T "%n - RemoteBox" %h:%p',
              HEARTBEAT       => 1,
              ADDADDITIONS    => 1,
              RDPAUTOOPEN     => 1,
              MSGLOGEXPAND    => 1,
              GUESTLISTEXPAND => 1,
              SSLVERIFY       => 0,
              DEFRDPPORTS     => '3389-4389',
              AUTOHINTDISP    => 0,
              AUTOHINTDISPX   => 800,
              AUTOHINTDISPY   => 600,
              AUTOHINTDISPD   => 32,
              STOPTYPE        => 'INSTANT');

sub rbprefs_get() {
    $gui{PREFSDIR} = Glib::get_user_config_dir();
    $gui{CONFIGFILE} = "$gui{PREFSDIR}/remotebox.conf";
    $gui{THUMBDIR} = "$gui{PREFSDIR}/remoteboxthumbs";

    if (open(PREFS, '<', $gui{CONFIGFILE})) {
        my @contents = <PREFS>;
        chomp(@contents);
        close(PREFS);
        foreach (@contents) {
            if ($_ =~ m/^URL=(.*)$/) { $prefs{URL}{$1} = 'URL'; }
            elsif ($_ =~ m/^USER=(.*)$/) { $prefs{USER}{$1} = 'USER'; }
            elsif ($_ =~ m/^(.*?)=(.*)$/) { $prefs{$1} = $2; }
        }
    }
    else { $prefs{URL}{qq[http://localhost:18083]} = 'URL'; } #Add a default URL
}

sub rbprefs_save() {
	mkdir($gui{PREFSDIR}, 0755) unless (-e $gui{PREFSDIR});

    if (open(PREFS, '>', $gui{CONFIGFILE})) {
        foreach my $key (keys %prefs) {
            if ($key eq 'URL') {
                foreach (keys %{$prefs{URL}}) { print PREFS "URL=$_\n"; }
            }
            elsif ($key eq 'USER') {
                foreach (keys %{$prefs{USER}}) { print PREFS "USER=$_\n"; }
            }
            else { print PREFS "$key=$prefs{$key}\n"; }
        }
        close(PREFS);
    }
    else { warn "Unable to save preferences: $gui{CONFIGFILE}\n"; }
}

sub show_dialog_editnat() {
    my $natref = &getsel_list_vbprefsnat();
    $gui{entryvbprefsNATName}->set_text($$natref{Name});
    $gui{entryvbprefsNATCIDR}->set_text(INATNetwork_getNetwork($$natref{INATNetwork}));
    $gui{checkbuttonvbprefsNATDHCP}->set_active(&bl(INATNetwork_getNeedDhcpServer($$natref{INATNetwork})));
    $gui{checkbuttonvbprefsNATipv6}->set_active(&bl(INATNetwork_getIPv6Enabled($$natref{INATNetwork})));
    $gui{checkbuttonvbprefsNATipv6route}->set_active(&bl(INATNetwork_getAdvertiseDefaultIPv6RouteEnabled($$natref{INATNetwork})));
    my $response = $gui{dialogNATDetails}->run;
    $gui{dialogNATDetails}->hide;

    if ($response eq 'ok') {
        INATNetwork_setNetworkName($$natref{INATNetwork}, $gui{entryvbprefsNATName}->get_text());
        my $cidr = $gui{entryvbprefsNATCIDR}->get_text();
        INATNetwork_setNetwork($$natref{INATNetwork}, $cidr) if (&valid_cidr($cidr));
        INATNetwork_setNeedDhcpServer($$natref{INATNetwork}, int($gui{checkbuttonvbprefsNATDHCP}->get_active()));
        INATNetwork_setIPv6Enabled($$natref{INATNetwork}, int($gui{checkbuttonvbprefsNATipv6}->get_active()));
        INATNetwork_setAdvertiseDefaultIPv6RouteEnabled($$natref{INATNetwork}, int($gui{checkbuttonvbprefsNATipv6route}->get_active()));
        &fill_list_vbprefsnat();
    }
}

sub show_dialog_prefs() {
    $gui{entryPrefsRDPClient}->set_text($prefs{RDPCLIENT});
    $gui{entryPrefsRDPPorts}->set_text($prefs{DEFRDPPORTS});
    $gui{checkbuttonPrefsRDPAuto}->set_active($prefs{RDPAUTOOPEN});
    $gui{checkbuttonPrefsHeartbeat}->set_active($prefs{HEARTBEAT});
    $gui{checkbuttonPrefsAddAdditions}->set_active($prefs{ADDADDITIONS});
    $gui{checkbuttonPrefsAutoHint}->set_active($prefs{AUTOHINTDISP});
    $gui{spinbuttonPrefsWidth}->set_value($prefs{AUTOHINTDISPX});
    $gui{spinbuttonPrefsHeight}->set_value($prefs{AUTOHINTDISPY});

    if ($prefs{AUTOHINTDISPD} == 8) { $gui{comboboxPrefsDepth}->set_active(0); }
    elsif ($prefs{AUTOHINTDISPD} == 16) { $gui{comboboxPrefsDepth}->set_active(1); }
    else { $gui{comboboxPrefsDepth}->set_active(2); }

    if ($prefs{STOPTYPE} eq 'ACPI') { $gui{comboboxPrefsDefaultStop}->set_active(1); }
    elsif ($prefs{STOPTYPE} eq 'STATE')  { $gui{comboboxPrefsDefaultStop}->set_active(2); }
    else { $gui{comboboxPrefsDefaultStop}->set_active(0); }

    my $response = $gui{dialogPrefs}->run;
    $gui{dialogPrefs}->hide;

    if ($response eq 'ok') {
        $prefs{RDPCLIENT} = $gui{entryPrefsRDPClient}->get_text();
        $prefs{DEFRDPPORTS} = $gui{entryPrefsRDPPorts}->get_text();
        $prefs{RDPAUTOOPEN} = int($gui{checkbuttonPrefsRDPAuto}->get_active());
        $prefs{HEARTBEAT} = int($gui{checkbuttonPrefsHeartbeat}->get_active());
        $prefs{ADDADDITIONS} = int($gui{checkbuttonPrefsAddAdditions}->get_active());
        $prefs{STOPTYPE} = &getsel_combo($gui{comboboxPrefsDefaultStop}, 1);
        $prefs{AUTOHINTDISP} = int($gui{checkbuttonPrefsAutoHint}->get_active());
        $prefs{AUTOHINTDISPX} = int($gui{spinbuttonPrefsWidth}->get_value());
        $prefs{AUTOHINTDISPY} = int($gui{spinbuttonPrefsHeight}->get_value());
        $prefs{AUTOHINTDISPD} = &getsel_combo($gui{comboboxPrefsDepth}, 1);
        &rbprefs_save();
    }
}

# Clears the connection history so the connect dialog will be empty
sub show_dialog_clearhistory() {
    my $response = $gui{messagedialogClearHistory}->run;
    $gui{messagedialogClearHistory}->hide;

    if ($response eq 'ok') {
        $prefs{USER} = ();
        $prefs{URL} = ();
        &rbprefs_save();
        &addrow_log('Cleared connection history.');
    }
}

# Shows the VirtualBox preferences dialog
sub show_dialog_vbprefs() {
    my $vhost = &vhost();
    $gui{entryVBPrefsGenMachineFolder}->set_text($$vhost{machinedir});
    $gui{entryVBPrefsGenAutostartDBFolder}->set_text($$vhost{autostartdb});
    $gui{entryVBPrefsGenVRDPAuth}->set_text($$vhost{vrdelib});
    $gui{checkbuttonVBPrefsHWExclusive}->set_active(&bl($$vhost{hwexclusive}));
    &fill_list_vbprefshon();
    &fill_list_vbprefsnat();
    $gui{dialogVBPrefs}->run();
    $gui{dialogVBPrefs}->hide();

    my $machinedir = $gui{entryVBPrefsGenMachineFolder}->get_text();
    my $autostartdir = $gui{entryVBPrefsGenAutostartDBFolder}->get_text();
    my $vrdelib = $gui{entryVBPrefsGenVRDPAuth}->get_text();
    $vrdelib =~ s/\.dll$//i;
    $vrdelib =~ s/\.so$//i;
    ISystemProperties_setDefaultMachineFolder($$vhost{ISystemProperties}, $machinedir) if ($machinedir ne $$vhost{machinedir});
    ISystemProperties_setVRDEAuthLibrary($$vhost{ISystemProperties}, $vrdelib) if ($vrdelib ne $$vhost{vrdelib});
    ISystemProperties_setAutostartDatabasePath($$vhost{ISystemProperties}, $autostartdir) if ($autostartdir ne $$vhost{autostartdb});
    if ($gui{checkbuttonVBPrefsHWExclusive}->get_active()) { ISystemProperties_setExclusiveHwVirt($$vhost{ISystemProperties}, 'true'); }
    else { ISystemProperties_setExclusiveHwVirt($$vhost{ISystemProperties}, 'false'); }
    &clr_vhost(); # VB Prefs changes can potentially alter vhost values, so clear them to be repopulated
}

sub show_dialog_edithon() {
    my $ifref = &getsel_list_vbprefshon();
    my @DHCPServers = IVirtualBox_getDHCPServers($gui{websn});
    my $IDHCPServer;

    # Check if it has a DCHP server associated with it
    foreach my $server (@DHCPServers) {
        if ("HostInterfaceNetworking-$$ifref{Name}" eq IDHCPServer_getNetworkName($server)) {
            $IDHCPServer = $server;
            last;
        }
    }

    # If not DHCP server, then create one.
    $IDHCPServer = IVirtualBox_createDHCPServer($gui{websn}, "HostInterfaceNetworking-$$ifref{Name}") if (!$IDHCPServer);
    $gui{entryHONAddress}->set_text(IHostNetworkInterface_getIPAddress($$ifref{IHostNetworkInterface}));
    $gui{entryHONNetmask}->set_text(IHostNetworkInterface_getNetworkMask($$ifref{IHostNetworkInterface}));
    $gui{entryHON6Address}->set_text(IHostNetworkInterface_getIPV6Address($$ifref{IHostNetworkInterface}));
    $gui{entryHON6Netmask}->set_text(IHostNetworkInterface_getIPV6NetworkMaskPrefixLength($$ifref{IHostNetworkInterface}));
    $gui{checkbuttonHONDHCP}->set_active(&bl(IDHCPServer_getEnabled($IDHCPServer)));
    $gui{tableHONDHCP}->set_sensitive($gui{checkbuttonHONDHCP}->get_active()); # Ghost/Unghost other widgets based on dhcp enabled
    $gui{entryHONServerAddress}->set_text(IDHCPServer_getIPAddress($IDHCPServer));
    $gui{entryHONServerMask}->set_text(IDHCPServer_getNetworkMask($IDHCPServer));
    $gui{entryHONLBound}->set_text(IDHCPServer_getLowerIP($IDHCPServer));
    $gui{entryHONUBound}->set_text(IDHCPServer_getUpperIP($IDHCPServer));
    my $response = $gui{dialogHON}->run();
    $gui{dialogHON}->hide();

    if ($response eq 'ok') {
        IDHCPServer_setEnabled($IDHCPServer, $gui{checkbuttonHONDHCP}->get_active());
        my ($sip4, $sip4nm, $slb, $shb) = ($gui{entryHONServerAddress}->get_text(), $gui{entryHONServerMask}->get_text(), $gui{entryHONLBound}->get_text(), $gui{entryHONUBound}->get_text());
        IDHCPServer_setConfiguration($IDHCPServer, $sip4, $sip4nm, $slb, $shb) if (&valid_ipv4($sip4) and &valid_ipv4($sip4nm) and &valid_ipv4($slb) and &valid_ipv4($shb));

        my ($ip4, $ip4nm) = ($gui{entryHONAddress}->get_text(), $gui{entryHONNetmask}->get_text());
        IHostNetworkInterface_enableStaticIPConfig($$ifref{IHostNetworkInterface}, $ip4, $ip4nm) if (&valid_ipv4($ip4) and &valid_ipv4($ip4nm));

        IHostNetworkInterface_enableStaticIPConfigV6($$ifref{IHostNetworkInterface},
                                                     $gui{entryHON6Address}->get_text(),
                                                     $gui{entryHON6Netmask}->get_text());
        &fill_list_vbprefshon();
    }
}

sub vbprefs_reset() {
    my ($widget) = @_;
    my $ISystemProperties = IVirtualBox_getSystemProperties($gui{websn});

    if ($widget eq $gui{buttonVBPrefsGenMachineFolderReset}) {
        ISystemProperties_setDefaultMachineFolder($ISystemProperties, '');
        $gui{entryVBPrefsGenMachineFolder}->set_text(ISystemProperties_getDefaultMachineFolder($ISystemProperties));
    }
    elsif ($widget eq $gui{buttonVBPrefsGenDefVRDPAuthReset}) {
        ISystemProperties_setVRDEAuthLibrary($ISystemProperties, '');
        $gui{entryVBPrefsGenVRDPAuth}->set_text(ISystemProperties_getVRDEAuthLibrary($ISystemProperties));
    }
}

sub vbprefs_createhon() {
    my $IHost = IVirtualBox_getHost($gui{websn});
    my ($hostinterface, $IProgress) = IHost_createHostOnlyNetworkInterface($IHost);
    &show_progress_window($IProgress, 'Adding host only network');
    &fill_list_vbprefshon();
}

sub vbprefs_createnat() {
    IVirtualBox_createNATNetwork($gui{websn}, 'NatNetwork' . int(rand(9999)));
    &fill_list_vbprefsnat();
}

sub vbprefs_removehon() {
    my $ifref = &getsel_list_vbprefshon();
    my @DHCPServers = IVirtualBox_getDHCPServers($gui{websn});

    # Check if it has a DCHP server associated with it
    foreach my $server (@DHCPServers) {
        if ("HostInterfaceNetworking-$$ifref{Name}" eq IDHCPServer_getNetworkName($server)) {
            IVirtualBox_removeDHCPServer($gui{websn}, $server);
            last;
        }
    }

    my $IHost = IVirtualBox_getHost($gui{websn});
    IHost_removeHostOnlyNetworkInterface($IHost, $$ifref{Uuid});
    &fill_list_vbprefshon();
}

sub vbprefs_removenat() {
    my $natref = &getsel_list_vbprefsnat();
    IVirtualBox_removeNATNetwork($gui{websn}, $$natref{INATNetwork});
    &fill_list_vbprefsnat();
}

# Can't use getsel_list_vbprefsnat in here due to the way the signals are propagated
sub vbprefs_togglenat() {
    my ($widget, $path_str, $model) = @_;
    my $iter = $model->get_iter(Gtk2::TreePath->new_from_string($path_str));
    my $val = $model->get($iter, 0);
    my $INATNetwork = $model->get($iter, 2);
    INATNetwork_setEnabled($INATNetwork, !$val);
    &fill_list_vbprefsnat();
}

sub rbprefs_msglog() {
    my ($widget) = @_;
    $prefs{MSGLOGEXPAND} = int($widget->get_expanded()); # int forces 0 on undefined
    &rbprefs_save();
}

sub rbprefs_saveurluser() {
    $prefs{SAVEURLUSER} = $gui{checkbuttonConnectSave}->get_active();
    &rbprefs_save();
}

sub rbprefs_sslverify() {
    my $active = $gui{checkbuttonConnectSSL}->get_active();
    $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = $active;
    $prefs{SSLVERIFY} = $active;
    &rbprefs_save();
}

sub rbprefs_autohint() {
    if ($gui{checkbuttonPrefsAutoHint}->get_active()) {
        $gui{spinbuttonPrefsWidth}->set_sensitive(1);
        $gui{spinbuttonPrefsHeight}->set_sensitive(1);
        $gui{comboboxPrefsDepth}->set_sensitive(1);
    }
    else {
        $gui{spinbuttonPrefsWidth}->set_sensitive(0);
        $gui{spinbuttonPrefsHeight}->set_sensitive(0);
        $gui{comboboxPrefsDepth}->set_sensitive(0);
    }
}

sub vbprefs_hon_dhcp_toggle() {
    $gui{tableHONDHCP}->set_sensitive($gui{checkbuttonHONDHCP}->get_active()); # Ghost/Unghost other widgets based on dhcp enabled
}

1;